using NetTopologySuite.Algorithm;
using NetTopologySuite.Geometries;
using NUnit.Framework;

namespace NetTopologySuite.Tests.NUnit.Algorithm
{
    [TestFixture]
    public class RobustLineIntersectorTest
    {
        RobustLineIntersector i = new RobustLineIntersector();

        [Test]
        public void Test2Lines() {
            var i = new RobustLineIntersector();
            var p1 = new Coordinate(10, 10);
            var p2 = new Coordinate(20, 20);
            var q1 = new Coordinate(20, 10);
            var q2 = new Coordinate(10, 20);
            var x = new Coordinate(15, 15);
            i.ComputeIntersection(p1, p2, q1, q2);
            Assert.AreEqual(LineIntersector.PointIntersection, i.IntersectionNum);
            Assert.AreEqual(1, i.IntersectionNum);
            Assert.AreEqual(x, i.GetIntersection(0));
            Assert.IsTrue(i.IsProper);
            Assert.IsTrue(i.HasIntersection);
        }

        [Test]
        public void TestCollinear1() {
            var i = new RobustLineIntersector();
            var p1 = new Coordinate(10, 10);
            var p2 = new Coordinate(20, 10);
            var q1 = new Coordinate(22, 10);
            var q2 = new Coordinate(30, 10);
            i.ComputeIntersection(p1, p2, q1, q2);
            Assert.AreEqual(LineIntersector.NoIntersection, i.IntersectionNum);
            Assert.IsTrue(!i.IsProper);
            Assert.IsTrue(!i.HasIntersection);
        }

        [Test]
        public void TestCollinear2() {
            var i = new RobustLineIntersector();
            var p1 = new Coordinate(10, 10);
            var p2 = new Coordinate(20, 10);
            var q1 = new Coordinate(20, 10);
            var q2 = new Coordinate(30, 10);
            i.ComputeIntersection(p1, p2, q1, q2);
            Assert.AreEqual(LineIntersector.PointIntersection, i.IntersectionNum);
            Assert.IsTrue(!i.IsProper);
            Assert.IsTrue(i.HasIntersection);
        }

        [Test]
        public void TestCollinear3() {
            var i = new RobustLineIntersector();
            var p1 = new Coordinate(10, 10);
            var p2 = new Coordinate(20, 10);
            var q1 = new Coordinate(15, 10);
            var q2 = new Coordinate(30, 10);
            i.ComputeIntersection(p1, p2, q1, q2);
            Assert.AreEqual(LineIntersector.CollinearIntersection, i.IntersectionNum);
            Assert.IsTrue(!i.IsProper);
            Assert.IsTrue(i.HasIntersection);
        }

        [Test]
        public void TestCollinear4() {
            var i = new RobustLineIntersector();
            var p1 = new Coordinate(30, 10);
            var p2 = new Coordinate(20, 10);
            var q1 = new Coordinate(10, 10);
            var q2 = new Coordinate(30, 10);
            i.ComputeIntersection(p1, p2, q1, q2);
            Assert.AreEqual(LineIntersector.CollinearIntersection, i.IntersectionNum);
            Assert.IsTrue(i.HasIntersection);
        }

        [Test]
        public void TestEndpointIntersection() {
            i.ComputeIntersection(new Coordinate(100, 100), new Coordinate(10, 100),
            new Coordinate(100, 10), new Coordinate(100, 100));
            Assert.IsTrue(i.HasIntersection);
            Assert.AreEqual(1, i.IntersectionNum);
        }

        [Test]
        public void TestEndpointIntersection2() {
            i.ComputeIntersection(new Coordinate(190, 50), new Coordinate(120, 100),
            new Coordinate(120, 100), new Coordinate(50, 150));
            Assert.IsTrue(i.HasIntersection);
            Assert.AreEqual(1, i.IntersectionNum);
            Assert.AreEqual(new Coordinate(120, 100), i.GetIntersection(1));
        }

        [Test]
        public void TestOverlap() {
            i.ComputeIntersection(new Coordinate(180, 200), new Coordinate(160, 180),
            new Coordinate(220, 240), new Coordinate(140, 160));
            Assert.IsTrue(i.HasIntersection);
            Assert.AreEqual(2, i.IntersectionNum);
        }

        [Test]
        public void TestIsProper1() {
            i.ComputeIntersection(new Coordinate(30, 10), new Coordinate(30, 30),
            new Coordinate(10, 10), new Coordinate(90, 11));
            Assert.IsTrue(i.HasIntersection);
            Assert.AreEqual(1, i.IntersectionNum);
            Assert.IsTrue(i.IsProper);
        }

        [Test]
        public void TestIsProper2() {
            i.ComputeIntersection(new Coordinate(10, 30), new Coordinate(10, 0),
            new Coordinate(11, 90), new Coordinate(10, 10));
            Assert.IsTrue(i.HasIntersection);
            Assert.AreEqual(1, i.IntersectionNum);
            Assert.IsTrue(!i.IsProper);
        }

        [Test]
        public void TestIsCCW() {
            Assert.AreEqual(OrientationIndex.CounterClockwise, Orientation.Index(
            new Coordinate(-123456789, -40),
            new Coordinate(0, 0),
            new Coordinate(381039468754763d, 123456789)));
        }

        [Test]
        public void TestIsCCW2() {
            Assert.AreEqual(OrientationIndex.Collinear, Orientation.Index(
            new Coordinate(10, 10),
            new Coordinate(20, 20),
            new Coordinate(0, 0)));
        }

        [Test]
        public void TestA() {
            var p1 = new Coordinate(-123456789, -40);
            var p2 = new Coordinate(381039468754763d, 123456789);
            var q  = new Coordinate(0, 0);
            var l = new GeometryFactory().CreateLineString(new Coordinate[] {p1, p2});
            var p = new GeometryFactory().CreatePoint(q);
            Assert.AreEqual(false, l.Intersects(p));
            Assert.AreEqual(false, PointLocation.IsOnLine(q, new Coordinate[] { p1, p2 }));
            Assert.AreEqual(OrientationIndex.Clockwise, Orientation.Index(p1, p2, q));
        }

    }
}