﻿namespace NetTopologySuite.Geometries
{
    /// <summary>
    /// Identifies the different well-supported components of coordinate values.
    /// <para>
    /// All supported coordinates define values for at least the X and the Y components.
    /// </para>
    /// <para>
    /// The first 16 spatial and 16 measure dimensions may also be accessed this way.
    /// </para>
    /// </summary>
    public enum Ordinate
    {
        /// <summary>
        /// The 1st spatial dimension.
        /// </summary>
        Spatial1,

        /// <summary>
        /// The "X" ordinate.
        /// </summary>
        X = Spatial1,

        /// <summary>
        /// The 2nd spatial dimension.
        /// </summary>
        Spatial2,

        /// <summary>
        /// The "Y" ordinate.
        /// </summary>
        Y = Spatial2,

        /// <summary>
        /// The 3rd spatial dimension.
        /// </summary>
        Spatial3,

        /// <summary>
        /// The "Z" ordinate.
        /// </summary>
        Z = Spatial3,

        /// <summary>
        /// The 4th spatial dimension.
        /// </summary>
        Spatial4,

        /// <summary>
        /// The 5th spatial dimension.
        /// </summary>
        Spatial5,

        /// <summary>
        /// The 6th spatial dimension.
        /// </summary>
        Spatial6,

        /// <summary>
        /// The 7th spatial dimension.
        /// </summary>
        Spatial7,

        /// <summary>
        /// The 8th spatial dimension.
        /// </summary>
        Spatial8,

        /// <summary>
        /// The 9th spatial dimension.
        /// </summary>
        Spatial9,

        /// <summary>
        /// The 10th spatial dimension.
        /// </summary>
        Spatial10,

        /// <summary>
        /// The 11th spatial dimension.
        /// </summary>
        Spatial11,

        /// <summary>
        /// The 12th spatial dimension.
        /// </summary>
        Spatial12,

        /// <summary>
        /// The 13th spatial dimension.
        /// </summary>
        Spatial13,

        /// <summary>
        /// The 14th spatial dimension.
        /// </summary>
        Spatial14,

        /// <summary>
        /// The 15th spatial dimension.
        /// </summary>
        Spatial15,

        /// <summary>
        /// The 16th spatial dimension.
        /// </summary>
        Spatial16,

        /// <summary>
        /// The 1st measure dimension.
        /// </summary>
        Measure1,

        /// <summary>
        /// The "M" ordinate.
        /// </summary>
        M = Measure1,

        /// <summary>
        /// The 2nd measure dimension.
        /// </summary>
        Measure2,

        /// <summary>
        /// The 3rd measure dimension.
        /// </summary>
        Measure3,

        /// <summary>
        /// The 4th measure dimension.
        /// </summary>
        Measure4,

        /// <summary>
        /// The 5th measure dimension.
        /// </summary>
        Measure5,

        /// <summary>
        /// The 6th measure dimension.
        /// </summary>
        Measure6,

        /// <summary>
        /// The 7th measure dimension.
        /// </summary>
        Measure7,

        /// <summary>
        /// The 8th measure dimension.
        /// </summary>
        Measure8,

        /// <summary>
        /// The 9th measure dimension.
        /// </summary>
        Measure9,

        /// <summary>
        /// The 10th measure dimension.
        /// </summary>
        Measure10,

        /// <summary>
        /// The 11th measure dimension.
        /// </summary>
        Measure11,

        /// <summary>
        /// The 12th measure dimension.
        /// </summary>
        Measure12,

        /// <summary>
        /// The 13th measure dimension.
        /// </summary>
        Measure13,

        /// <summary>
        /// The 14th measure dimension.
        /// </summary>
        Measure14,

        /// <summary>
        /// The 15th measure dimension.
        /// </summary>
        Measure15,

        /// <summary>
        /// The 16th measure dimension.
        /// </summary>
        Measure16,
    }
}
